--[[
    MyRolePlay 4 (C) 2010-2022 Etarna Moonshyne <etarna@moonshyne.org>, Katorie, Meorawr
    Licensed under GNU General Public Licence version 2 or, at your option, any later version

    UI_Tab.lua - Functions to add the MyRolePlay tab to the CharacterFrame, and add the MRP "button" to the CharacterFrame in Vanilla, BC, and WotLK.
]]

local L = mrp.L

local tinsert = tinsert

function mrp:AddMRPTab()
    if not mrp.Tabname then
        -- Create the button for the tab
        -- Do NOT change the frame name, it's magic, see FrameXML/UIPanelTemplates.lua
        mrp.oldNumTabs = CharacterFrame.numTabs
        mrp.Tabname = "CharacterFrameTab"..CharacterFrame.numTabs+1
        local tab = CreateFrame( "Button", mrp.Tabname, CharacterFrame, DoesTemplateExist("CharacterFrameTabTemplate") and "CharacterFrameTabTemplate" or "CharacterFrameTabButtonTemplate", CharacterFrame.numTabs+1 )
		if mrp:IsMainlineClient() or mrp:IsCataClient() then
            -- In vanilla / BC / WotLK we need a shorter tab for hunters and warlocks due to space issues.
            tab:SetText( L["tabtitle"] )
        else
            tab:SetText( L["tabtitleshort"] )
        end

        tab:SetPoint( "LEFT", "CharacterFrameTab"..CharacterFrame.numTabs, "RIGHT", 3, 0 )
        tab:SetScript( "OnEnter", function(self)
            GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
            GameTooltip:SetText( L["editor_mrptab_tt"], 1.0, 1.0, 1.0 )
        end )
        tab:SetScript( "OnLeave", GameTooltip_Hide )
        tab:SetScript( "OnClick", function(self)
            ToggleCharacter( "MyRolePlayCharacterFrame" )
            PlaySound(841) -- I have no idea *why* it does this twice, but it does sound louder when you do
        end )

        -- Ellypse: Sync tab ID and frame ID to have the correct selected state behavior in character frame
        MyRolePlayCharacterFrame:SetID(CharacterFrame.numTabs+1);

        -- Add our subframe to the list of frames (Taints CHARACTERFRAME_SUBFRAMES as of 8.2)
		-- CharacterFrame_ShowSubFrames doesn't seem to exist in only Cata, so working around that below.
        if CharacterFrame_ShowSubFrame then
			hooksecurefunc("CharacterFrame_ShowSubFrame", function(frameName)
				local show = (frameName == "MyRolePlayCharacterFrame");
				MyRolePlayCharacterFrame:SetShown(show);
			end)
		else
			hooksecurefunc(CharacterFrame, "ShowSubFrame", function(_, frameName)
				local show = (frameName == "MyRolePlayCharacterFrame");
				MyRolePlayCharacterFrame:SetShown(show);
			end)
		end;

        -- Increase the number of tabs to match (Taints CharacterFrame.numTabs as of 8.2)
        --PanelTemplates_SetNumTabs( CharacterFrame, CharacterFrame.numTabs+1 )
        hooksecurefunc("PanelTemplates_UpdateTabs", function(frame)
            if tab.isDisabled then
                PanelTemplates_SetDisabledTabState(tab);
            elseif frame.selectedTab == (mrp.oldNumTabs + 1) then
                PanelTemplates_SelectTab(tab);
            else
                PanelTemplates_DeselectTab(tab);
            end
        end);

        -- Fix issues with the appropriate icon not showing when the UI
        -- is initially opened sometimes. CharacterFrame_UpdatePortrait doesn't exist on BC, Classic, WotLK or Cata.
		if mrp:IsMainlineClient() then
			hooksecurefunc("CharacterFrame_UpdatePortrait", function()
				if MyRolePlayCharacterFrame:IsShown() then
					mrp:UpdateCharacterFrame();
				end
			end);
		end

        -- Nudge the positioning/resizing so the game (hopefully) fits the tabs under the window again
        PanelTemplates_TabResize( _G[mrp.Tabname], 0 )
        CharacterFrame_TabBoundsCheck( _G[mrp.Tabname] )
    else
        _G[mrp.Tabname]:Show()
    end
end

function mrp:RemoveMRPTab()
    if mrp.Tabname then
        _G[mrp.Tabname]:Hide()
    end
end

-- Tab size/location fixing function to re-anchor the tabs and fit them under the window
-- Fixed for 4.3: updated from Blizzard's CharacterFrame.lua (theirs is still hardcoded).
CharacterFrame_TabBoundsCheck = nil -- Kill and garbage-collect Blizzard's FrameXML version

local function CompareFrameSize(frame1, frame2)
    return frame1:GetWidth() > frame2:GetWidth();
end

local CharTabtable = {};

function CharacterFrame_TabBoundsCheck(self)
    if ( string.sub(self:GetName(), 1, 17) ~= "CharacterFrameTab" ) then
        return;
    end

    local totalSize = 60
    local previouslyshowntab = CharacterFrameTab1
    for i=1, CharacterFrame.numTabs do
		if not mrp:IsMainlineClient() then -- This was removed in Dragonflight prepatch 10.0 with the changes to the CharacterFrameTabTemplate.
			_G["CharacterFrameTab"..i.."Text"]:SetWidth(0)
		end
        PanelTemplates_TabResize(_G["CharacterFrameTab"..i], 0)
        if i>1 then
            _G["CharacterFrameTab"..i]:SetPoint( "LEFT", previouslyshowntab, "RIGHT", -15, 0 )
        end
        if _G["CharacterFrameTab"..i]:IsShown() then
            previouslyshowntab =  _G["CharacterFrameTab"..i]
            totalSize = totalSize + previouslyshowntab:GetWidth()
        end
    end

    if mrp.Tabname then
        if not CharacterFrameTab4:IsShown() then
            if not CharacterFrameTab3:IsShown() then
                if not CharacterFrameTab2:IsShown() then
                    _G[mrp.Tabname]:SetPoint( "LEFT", "CharacterFrameTab1", "RIGHT", -15, 0 )
                else
                    _G[mrp.Tabname]:SetPoint( "LEFT", "CharacterFrameTab2", "RIGHT", -15, 0 )
                end
            else
                _G[mrp.Tabname]:SetPoint( "LEFT", "CharacterFrameTab3", "RIGHT", -15, 0 )
            end
        end
    end

    local threshold = 435
    local diff = totalSize - threshold

    if ( totalSize > threshold ) then
        --Find the biggest tab
        for i=1, CharacterFrame.numTabs do
            CharTabtable[i]=_G["CharacterFrameTab"..i];
        end
        table.sort(CharTabtable, CompareFrameSize);

        local i=1;
        while ( diff > 0 and i <= CharacterFrame.numTabs) do
            local tabText = _G[CharTabtable[i]:GetName().."Text"]
            local change = min(10, diff);
            tabText:SetWidth(tabText:GetWidth() - change);
            diff = diff - change;
            PanelTemplates_TabResize(CharTabtable[i], -change, nil, 8-change, 88);
            i = i+1;
        end
    end
end

-- Create an alternative MRP editor button, positioned at the top right of the character frame for classic versions of the game as the tab typically used in retail will not fit in the narrow frame. DOES NOT APPLY TO RETAIL.
if not mrp:IsMainlineClient() then
	local mrpEditorButton = CreateFrame( "Button", "MyRolePlayEditor_OpenEditorButton", CharacterFrame, "UIPanelButtonTemplate" )
	mrpEditorButton:SetPoint( "RIGHT", CharacterFrameCloseButton, "LEFT", 5, 0 )
	mrpEditorButton:SetText( L["MRP"] )
	mrpEditorButton:SetWidth( 34 )
	mrpEditorButton:SetHeight( 23 )
	mrpEditorButton:SetFrameStrata("HIGH");
	mrpEditorButton:SetScript( "OnEnter", function(self)
		GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
		GameTooltip:SetText( L["Edit your roleplaying profiles."], 1.0, 1.0, 1.0 )
	end )
	mrpEditorButton:SetScript( "OnClick", function(self)
		ToggleCharacter( "MyRolePlayCharacterFrame" )
        PlaySound(841) -- I have no idea *why* it does this twice, but it does sound louder when you do
		mrpEditorButton:Hide();
    end )
	mrpEditorButton:SetScript( "OnLeave", GameTooltip_Hide )
end
